# -*- coding: UTF-8 -*-

from xml.dom.minidom import Document
import os
import re
import sys

class XmlConfigWriter(object):
    """
    Создает файлы настроек для оборудования в формате xml
    """
    def __init__(self):
        self.ports = {}
        self.factories = {}
        self.devices = {}
        self.pluginsPath = '/linuxcash/cash/conf/drivers'

    def saveXml(self, fileName, document):
        xmlfile = open(os.path.join(self.pluginsPath, fileName), 'w')
        xmlstr = re.sub("\s+</value>","</value>",re.sub("<value>\s+","<value>",document.toprettyxml(indent = "    ")))
        xmlfile.write(xmlstr)
        xmlfile.close()

    def addFactory(self, libName):
        if libName in self.factories:
            return self.factories[libName]

        factoryName = libName + "_factory"
        doc = Document()
        objects = doc.createElement("objects")
        doc.appendChild(objects)
        object = doc.createElement("object")
        if os.path.exists(os.path.join(self.pluginsPath, factoryName + '.xml')):
            factoryName = factoryName + '_1'
        object.setAttribute("id", factoryName)
        object.setAttribute("plugin","/linuxcash/cash/drivers/lib" + libName + ".so")
        objects.appendChild(object)
        self.saveXml(factoryName + '.xml', doc)
        self.factories[libName] = factoryName
        return factoryName

    def addSerial(self, port):
        if port in self.ports:
            return self.ports[port]
        
        options = {'device': port}
        serialDevice = self.addDevice('serial', options)
        self.ports[port] = serialDevice
        return serialDevice

    def addDevice(self, libName, options):
        if libName not in self.devices:
            self.devices[libName] = 0
        else:
            self.devices[libName] += 1
        
        factoryName = self.addFactory(libName)
        deviceName = "%s_%s" % (libName, self.devices[libName])
        doc = Document()
        objects = doc.createElement("objects")
        doc.appendChild(objects)
        object = doc.createElement("object")
        object.setAttribute("id", deviceName)
        object.setAttribute("factory-object", factoryName)
        object.setAttribute("factory-method", "getObject")
        object.setAttribute("init-method", "init")
        objects.appendChild(object)

        for key in options:
            if isinstance(options[key], dict):
                optValue = options[key]['value']
                optType = options[key]["type"]
            else:
                optValue = options[key]
                optType = 'default'
            if not optValue:
                continue

            if optType == "serialRef":
                property = doc.createElement("property")
                property.setAttribute("name", key)
                object.appendChild(property)
                ref = doc.createElement("ref")
                ref.setAttribute("object", self.addSerial(optValue))
                property.appendChild(ref)
            elif optType == "deviceRef":
                property = doc.createElement("property")
                property.setAttribute("name", key)
                object.appendChild(property)
                ref = doc.createElement("ref")
                ref.setAttribute("object", optValue)
                property.appendChild(ref)
            else:
                if optType == "bool":
                    if optValue.lower() in ["ok", "yes", "enable", "1", "allow", "true"]:
                        optValue = "true"
                    else:
                        optValue = "false"
                property = doc.createElement("property")
                property.setAttribute("name", key)
                object.appendChild(property)
                value = doc.createElement("value")
                property.appendChild(value)
                vtext = doc.createTextNode(optValue)
                value.appendChild(vtext)

        self.saveXml(deviceName + '.xml', doc)
        return deviceName
